"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const DataLoader = require('dataloader');
class MongoEntity {
    constructor(connector, collectionName) {
        this.connector = connector;
        this._collectionName = collectionName;
    }
    get collection() {
        if (!this._collection) {
            this._collection = this.connector.collection(this._collectionName);
        }
        return this._collection;
    }
    clearUpdateCaches(selector) {
        if (this._singleLoader) {
            if (selector._id) {
                this._singleLoader.clear(selector._id);
            }
            else {
                this._singleLoader.clearAll();
            }
        }
    }
    clearInsertCaches(selector) {
        if (this._multiLoader) {
            this._multiLoader.clearAll();
        }
    }
    find(selector, fields, skip, limit, timeout) {
        return this.collection.find(selector, fields, skip, limit, timeout);
    }
    findOne(selector, options) {
        return this.collection.findOne(selector, options);
    }
    findOneCachedById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._singleLoader) {
                this._singleLoader = new DataLoader((keys) => {
                    return Promise.all(keys.map((loadId) => __awaiter(this, void 0, void 0, function* () {
                        return yield this.collection.findOne({ _id: loadId });
                    })));
                });
            }
            return this._singleLoader.load(id);
        });
    }
    findManyCached() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._multiLoader) {
                this._multiLoader = new DataLoader((param) => {
                    return Promise.all([this.collection.find().toArray()]);
                });
            }
            return this._multiLoader.load('ALL');
        });
    }
    insert(document) {
        this.clearInsertCaches(document);
        return this.collection.insertOne(document);
    }
    update(selector, update, options) {
        this.clearUpdateCaches(selector);
        return this.collection.updateOne(selector, update, options);
    }
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = MongoEntity;
